#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "lcd.h"
#include "DHT.h"
#include "usart.h"

double temp[1], hum[1];
char door, motion;
char lamp1, lamp2, lamp3, fan;
char toSend = 0;
int fanLimit = 20;

void Init_pins()
{
    DDRA &= ~(1 << PA0);
	PORTA |= (1 << PA0);

    DDRA &= ~(1 << PA1);
	PORTA |= (1 << PA1);

	DDRA &= ~(1 << PA2);
	PORTA |= (1 << PA2);

	DDRA &= ~(1 << PA3);
	PORTA |= (1 << PA3);

    DDRA &= ~(1 << PA4);
	PORTA |= (1 << PA4);

	DDRB &= ~(1 << PB1);
	PORTB |= (1 << PB1);

	DDRD |= (1 << PD5) | (1 << PD6);

	PCICR |= (1 << PCIE0);
    PCMSK0 |= (1 << PCINT0) | (1 << PCINT1) | (1 << PCINT2) | (1 << PCINT3) | (1 << PCINT4);
}

void printData ()
{
	char line[21];

	LCD_clear_all();
	sprintf(line, "T:%.1lf  | U:%.1lf", temp[0], hum[0]);
	LCD_printAt(0x80, line);
	if (door) {
		sprintf(line, "Door open");
	} else {
		sprintf(line, "Door closed");
	}
	LCD_printAt(0xC0, line);
	if (motion) {
		sprintf(line, "Motion detected");
	} else {
		sprintf(line, "No motion");
	}
	LCD_printAt(0x94, line);
	sprintf(line, "Relay: O O O O");
	if (lamp1) line[7]='X';
	if (lamp2) line[9]='X';
	if (lamp3) line[11]='X';
	if (fan)   line[13]='X';
	LCD_printAt(0xD4, line);
}

ISR (PCINT0_vect) {
	if (PINA & (1 << PA1)) {
		lamp1 = 1 - lamp1;
	}

    if (PINA & (1 << PA2)) {
		lamp2 = 1 - lamp2;
	}

    if (PINA & (1 << PA3)) {
		lamp3 = 1 - lamp3;
	}

	if (PINA & (1 << PA4)) {
		fanLimit++;
		if (fanLimit > 28) {
			fanLimit = 16;
		}
	}
}

ISR (USART0_RX_vect){
	char a = UDR0;
	// Go forward!
	if (a =='1'){
		lamp1 = 1 - lamp1;
	} else if (a == '2') {
		lamp2 = 1 - lamp2;
	} else if (a == '3') {
		lamp3 = 1 - lamp3;
	} else if (a == 'f') {
		fan = 1 - fan;
	} else if (a == 'd') {
		if (door){
			toSend = 'o';
		} else {
			toSend =  ('c');
		}
	} else if (a == 'm') {
		if (motion){
			toSend = ('Y');
		} else {
			toSend = ('N');
		}
	} else if (a == '9') {
		if (lamp1){
			toSend = ('X');
		} else {
			toSend = ('O');
		}
	} else if (a == '8') {
		if (lamp2){
			toSend = ('X');
		} else {
			toSend = ('O');
		}
	} else if (a == '7') {
		if (lamp3){
			toSend = ('X');
		} else {
			toSend = ('O');
		}
	} else if (a == '6') {
		if (fan){
			toSend = ('X');
		} else {
			toSend = ('O');
		}
	} else if (a == -31) {
		// do nothing, it is end of transmision
		if (toSend != 0) {
			USART0_transmit(toSend);
			toSend = 0;
		}
	}
}

int main (void)
{
    Init_pins();

	// IO_init();
	LCD_init();

    DHT_Setup();

    USART0_init();

	sei();

	for(;;)
	{
		door = motion = 0;
		if (PINA & (1 << PA0)) {
			door = 1;
		}

		if (PINB & (1 << PB1)) {
			motion = 1;
		}

		DHT_Read(temp, hum);

		if (DHT_Status_check()) {
			temp[0] = 1.0;
			hum[0] = 1.0;
		}

		if (fanLimit > temp[0]) {
			fan = 1;
		}

		if (!lamp1) {
			PORTD &= ~(1 << PD6);
		} else {
			PORTD |= (1 << PD6);
		}

		if (!lamp2) {
			PORTD &= ~(1 << PD4);
		} else {
			PORTD |= (1 << PD4);
		}

		if (!lamp3) {
			PORTD &= ~(1 << PD3);
		} else {
			PORTD |= (1 << PD3);
		}

		if (!fan) {
			PORTD &= ~(1 << PD5);
		} else {
			PORTD |= (1 << PD5);
		}

		printData();
		_delay_ms(2000);

	}

	return 0;
}
